from __future__ import annotations
from imports import k8s
from cdk8s import Chart
from typing import Mapping


from kubeasy_sdk.utils.resource import Rendered


class Volume(Rendered):
  def __init__(self, name: str, labels: Mapping[str, str] = None):
    func_locals = dict(locals())
    del func_locals['self']
    super().__init__(**func_locals)

    self.name = name
    self.labels = labels

  def render_k8s_resource(self, chart: Chart) -> k8s.Volume:
    pass


class EmptyDir(Volume):
  def __init__(self, name: str, size_limit: str, use_memory: bool):
    super().__init__(name)
    self.use_memory = use_memory
    self.size_limit = size_limit
    self.medium_map = {False: None, True: 'memory'}

  def render_k8s_resource(self, chart: Chart) -> k8s.Volume:
    volume_size = k8s.Quantity.from_string(self.size_limit)
    volume_source = k8s.EmptyDirVolumeSource(size_limit=volume_size, medium=self.medium_map[self.use_memory])
    return k8s.Volume(name=self.name, empty_dir=volume_source)


class ConfigMap(Volume):
  def __init__(self, name: str, config_name: str):
    super().__init__(name)
    self.config_name = config_name

  def render_k8s_resource(self, chart: Chart) -> k8s.Volume:
    volume_source = k8s.ConfigMapVolumeSource(name=self.config_name)
    return k8s.Volume(name=self.name, config_map=volume_source)



from __future__ import annotations

import yaml
import json

from cdk8s import App, Chart
from constructs import Construct
from kubeasy_sdk.utils.collections.chart_resource_collection import ChartResourceCollection
from kubeasy_sdk.utils.resource import Rendered
from kubeasy_sdk.deployment import Deployment
from kubeasy_sdk.container import Container
from kubeasy_sdk.service import Service, ServicePort, ServiceType
from kubeasy_sdk.ingress import Ingress, IngressPath
from kubeasy_sdk.volume import Volume, ConfigMap, EmptyDir
from typing import Mapping


class EasyChart(object):
  def __init__(self, name: str, namespace: str, environment: str, release: str):
    self.name = name
    self.namespace = namespace
    self.environment = environment
    self.release = release

    self.deployment = Deployment(name=self.name,
                                 namespace=self.namespace,
                                 environment=self.environment)

    self.service_collection = ChartResourceCollection()
    self.ingress_collection = ChartResourceCollection()

  def add_init_container(self, name: str, image: str, tag: str) -> Container:
    new_init_container = Container(name, image, tag)
    self.deployment.add_init_container(new_init_container)
    return new_init_container

  def add_container(self, name: str, image: str, tag: str) -> Container:
    new_container = Container(name, image, tag)
    self.deployment.add_container(new_container)
    return new_container

  def include_volume(self, name: str, labels: Mapping[str, str]) -> Volume:
    new_volume = Volume(name, labels)
    self.deployment.include_volume(new_volume)
    return new_volume

  def include_config_map(self, name: str, config_name: str) -> ConfigMap:
    new_config_map = ConfigMap(name=name, config_name=config_name)
    self.deployment.include_volume(volume=new_config_map)
    return new_config_map

  def add_empty_dir(self, name: str, size_limit: str, use_memory: bool = False) -> EmptyDir:
    new_empty_dir = EmptyDir(name=name, size_limit=size_limit, use_memory=use_memory)
    self.deployment.include_volume(volume=new_empty_dir)
    return new_empty_dir

  def add_service(self, service_name) -> Service:
    new_service = Service(name=service_name, deployment=self.deployment)
    self.service_collection.add_resource(new_service)
    return new_service

  def add_ingress(self, name: str, tls: bool = False, labels: dict = None) -> Ingress:
    new_ingress = Ingress(name, tls, labels)
    self.ingress_collection.add_resource(new_ingress)
    return new_ingress

  def render(self) -> list:
    app = App()
    combined_resource_collection = ChartResourceCollection.combine([self.service_collection, self.ingress_collection])

    chart_json = self.__EasyChart(app, self.deployment, combined_resource_collection).to_json()

    yaml_manifest = ""
    for manifest in chart_json:
      yaml_manifest += f"---\n{yaml.dump(yaml.load(json.dumps(manifest), Loader=yaml.FullLoader))}"
    return yaml_manifest

  class __EasyChart(Chart):
    def __init__(self, scope: Construct, chart_deployment: Deployment, chart_resources: ChartResourceCollection):
      super().__init__(scope, chart_deployment.name)
      self.name = chart_deployment.name
      self.scope = scope
      chart_deployment.render(self)
      chart_resources.render(self)


import os
from cdk8s import App, Chart

from celery import Celery

from kubeasy_sdk import EasyChart
from kubeasy_sdk.utils.collections.chart_resource_collection import ChartResourceCollection

builder_app = Celery('builder', broker=os.getenv('KUBEASY_BROKER_URI', 'pyamqp://guest@localhost//'))

@builder_app.task()
def build_chart(chart: EasyChart):
    app = App()
    combined_resource_collection = ChartResourceCollection.combine([chart.service_collection, chart.ingress_collection])
    return chart.__EasyChart(app, chart.deployment, combined_resource_collection).to_json()


from __future__ import annotations
from imports import k8s
from cdk8s import Chart

from kubeasy_sdk.container import Container
from kubeasy_sdk.utils.collections.containers import Containers
from kubeasy_sdk.utils.resource import Rendered
from kubeasy_sdk.volume import Volume
from kubeasy_sdk.utils.collections.volumes import Volumes
from kubeasy_sdk.utils.security import SecurityContext


class Deployment(Rendered):

  def __init__(self, name: str, namespace: str, environment: str, replicas: int = 1):
    func_locals = dict(locals())
    del func_locals['self']
    super().__init__(**func_locals)

    self.name = name
    self.namespace = namespace
    self.environment = environment
    self.replicas = replicas

    self.labels = {}
    self.match_labels = {}

    self.image_pull_policy = None
    self.image_pull_secret = None

    self.pod_fs_gid = None

    self.init_containers = Containers()
    self.containers = Containers()
    self.volumes = Volumes()

    # Security Context
    self.security_context = SecurityContext()

  def set_replicas(self, replicas: int) -> Deployment:
    self.replicas = replicas
    return self

  # Deployment Labels

  def set_labels(self, labels: dict[str]) -> Deployment:
    self.labels = labels
    return self

  def add_label(self, key: str, value: str) -> Deployment:
    self.labels[key] = value
    return self

  # Deployment Match Labels

  def set_match_labels(self, match_labels: dict[str]) -> Deployment:
    self.match_labels = match_labels
    return self

  def add_match_label(self, key: str, value: str) -> Deployment:
    self.match_labels[key] = value
    return self

  # === Security Settings ===

  # Image Policies

  def set_image_pull_policy(self, pull_policy: str) -> Deployment:
    self.image_pull_policy = pull_policy
    return self

  def set_image_pull_secret(self, pull_secret: str) -> Deployment:
    self.image_pull_secret = pull_secret
    return self

  def set_pod_fs_gid(self, pod_fs_gid: int) -> Deployment:
    self.pod_fs_gid = pod_fs_gid
    return self

  # Containers

  def add_container(self, container: Container) -> Container:
    self.containers.add_container(container)
    return container

  # Init Containers

  def add_init_container(self, container: Container) -> Container:
    self.init_containers.add_container(container)
    return container

  # Volume Mounts

  def include_volume(self, volume: Volume) -> Volume:
    self.volumes.add_volume(volume)
    return volume

  def render_k8s_resource(self, chart: Chart) -> Deployment:

    # Create the metadata and label selectors for the deployment
    object_meta = k8s.ObjectMeta(labels=self.labels)
    label_selector = k8s.LabelSelector(match_labels=self.match_labels)

    # Generate the podspec templates for the deployment
    podspec = k8s.PodSpec(init_containers=self.init_containers.render(chart),
                          containers=self.containers.render(chart),
                          volumes=self.volumes.render(chart))

    podspec_template = k8s.PodTemplateSpec(metadata=object_meta,
                                           spec=podspec)

    # Use the podspec to create the deployment spec before finally returning the completed K8s Deployment.
    deployment_spec = k8s.DeploymentSpec(replicas=self.replicas, selector=label_selector, template=podspec_template)
    k8s.Deployment(chart, 'deployment', metadata=k8s.ObjectMeta(name=self.name), spec=deployment_spec)
    return self



from __future__ import annotations
from imports import k8s
from cdk8s import Chart

import jmespath

from kubeasy_sdk.utils.networking.service_port import ServicePort
from kubeasy_sdk.utils.resource import Rendered


class IngressPath(Rendered):
  def __init__(self, service_name: str, backend_port: int, path: str = None):
    func_locals = dict(locals())
    del func_locals['self']
    super().__init__(**func_locals)

    self.name = service_name
    self.port = backend_port
    self.path = path

  def render_k8s_resource(self, chart: Chart) -> k8s.HttpIngressPath:
    ingress_backend = k8s.IngressBackend(service_name=self.name, service_port=k8s.IntOrString.from_number(self.port))
    return k8s.HttpIngressPath(backend=ingress_backend, path=self.path)


class Ingress(Rendered):
  def __init__(self, name: str, tls: bool, labels: dict = None):
    func_locals = dict(locals())
    del func_locals['self']
    super().__init__(**func_locals)

    self.name = name
    self.tls = tls
    self.labels = labels
    self.annotations = {}
    self.rules = []
    self.service_ports = []

  def set_labels(self, labels: dict) -> Ingress:
    self.labels = labels
    return self

  def add_labels(self, key: str, value: str) -> Ingress:
    self.labels[key] = value
    return self

  def annotate(self, key: str, value: str) -> Ingress:
    self.annotations[key] = value
    return self

  def __create_object_meta(self):
    return k8s.ObjectMeta(name=self.name, labels=self.labels, annotations=self.annotations)

  def __create_ingress_tls(self, tls_hosts: list = []):
    if self.tls:
      return k8s.IngressTls(hosts=tls_hosts, secret_name=None)  # TODO: Figure out the secret business
    else:
      return None

  def add_rule(self, host: str, backend_port: ServicePort, path: str = None):
    self.rules.append({"host": host, "path": path, "port": backend_port})

  def render_k8s_resource(self, chart: Chart) -> k8s.Ingress:
    ingress_meta = self.__create_object_meta()
    ingress_tls = self.__create_ingress_tls()

    host_list = jmespath.search("rules[].host", {"rules": self.rules})

    ingress_rules = []
    for host in host_list:
      backend_list = jmespath.search(f"rules[?host == '{host}'].port", {"rules": self.rules})

      for backend in backend_list:
        ingress_backend = k8s.IngressBackend(service_name=backend.service_name, service_port=k8s.IntOrString.from_number(backend.port))
        backend_paths = jmespath.search(f"rules[?host == '{host}'].path", {"rules": self.rules})

        http_ingress_paths = []
        if len(backend_paths) > 0:
          for path in backend_paths:
            http_ingress_paths.append(k8s.HttpIngressPath(backend=ingress_backend, path=path))
        else:
          http_ingress_paths.append(k8s.HttpIngressPath(backend=ingress_backend))

        ingress_rules.append(k8s.IngressRule(host=host, http=k8s.HttpIngressRuleValue(paths=http_ingress_paths)))

    ingress_spec = k8s.IngressSpec(rules=ingress_rules, tls=ingress_tls)
    return k8s.Ingress(scope=chart, name=self.name, metadata=ingress_meta, spec=ingress_spec)


class SimpleIngress(Ingress):

  def __init__(self, name: str, tls: bool, service_name: str, service_port: int, labels: dict = None):
    super().__init__(name, tls, labels)
    self.name = service_name
    self.port = service_port

  def render_k8s_resource(self, chart: Chart) -> k8s.Ingress:
    ingress_meta = self.__create_object_meta()
    ingress_tls = self.__create_ingress_tls([self.name])
    ingress_backend = k8s.IngressBackend(service_name=self.name, service_port=k8s.IntOrString.from_number(self.port))
    ingress_spec = k8s.IngressSpec(backend=ingress_backend, tls=ingress_tls)
    return k8s.Ingress(scop=chart, name=self.name, metadata=ingress_meta, spec=ingress_spec)



from __future__ import annotations
from enum import Enum

from imports import k8s
from cdk8s import Chart

from kubeasy_sdk.deployment import Deployment
from kubeasy_sdk.utils.resource import Rendered
from kubeasy_sdk.utils.networking.service_port import ServicePort
from kubeasy_sdk.utils.collections.service_ports import ServicePorts


class ServiceType(Enum):
  CLUSTERIP = "ClusterIP"
  LOADBALANCER = "LoadBalancer"
  NODEPORT = "NodePort"

  def k8s_name(self) -> str:
    return '{0}'.format(self.value)


class Service(Rendered):
  def __init__(self, name: str, deployment: Deployment):
    func_locals = dict(locals())
    del func_locals['self']
    super().__init__(**func_locals)

    self.name = name
    self.deployment = deployment

    self.environment = deployment.environment
    self.namespace = deployment.namespace
    self.labels = {}
    self.selector = {}
    self.service_type = ServiceType.CLUSTERIP
    self.ports = ServicePorts()

  def set_type(self, service_type:  ServiceType) -> Service:
    self.service_type = service_type
    return self

  def add_port(self, service_port: ServicePort) -> ServicePort:
    return self.ports.add_port(service_port).set_service_name(self.name)

  # Service Labels

  def set_labels(self, labels: dict[str]) -> Service:
    self.labels = labels
    return self

  def add_label(self, key: str, value: str) -> Service:
    self.labels[key] = value
    return self

  # Service Selectors

  def set_selectors(self, selectors: dict[str]) -> Service:
    self.selector = selectors
    return self

  def add_selector(self, selector_key: str, selector_value: str) -> Service:
    self.selector[selector_key] = selector_value
    return self

  def render_k8s_resource(self, chart: Chart) -> k8s.Service:
    service_ports = ServicePort.render_port_list(self.ports)
    svc_spec = k8s.ServiceSpec(type=self.service_type.k8s_name(), ports=service_ports, selector=self.selector)
    object_meta = k8s.ObjectMeta(name=self.name, labels=self.labels)
    return k8s.Service(scope=chart, name=self.name, metadata=object_meta, spec=svc_spec)







import setuptools
import os
from requirementslib import Lockfile

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kubeasy-py",
    version=os.getenv('KUBEASY_VERSION', "v0.0.0"),
    author="Dylan Turnbull",
    author_email="dylanturn@gmail.com",
    description="Kubernetes made easy!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dylanturn/kubeasy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires = Lockfile.load("Pipfile.lock").as_requirements()
)


from __future__ import annotations

from typing import Mapping

from imports import k8s
from cdk8s import Chart

from kubeasy_sdk.utils.collections.container_ports import ContainerPorts
from kubeasy_sdk.utils.request_limits import ContainerResources
from kubeasy_sdk.utils.security import SecurityContext
from kubeasy_sdk.utils.networking.container_port import ContainerPort
from kubeasy_sdk.utils.resource import Rendered


class Container(Rendered):

  def __init__(self, name: str, image: str, tag: str):
    self.name = name
    self.image = image
    self.image_pull_policy = "Always"
    self.image_pull_secret = None
    self.tag = tag
    self.ports = ContainerPorts()
    self.security_context = SecurityContext()
    self.resource_requirements = ContainerResources()

    self.command = []
    self.env_variables = {}

    self.liveness_probe_path = None
    self.liveness_probe_port = None
    self.readiness_probe_path = None
    self.readiness_probe_port = None

    self.volume_mounts = []

    func_locals = dict(locals())
    del func_locals['self']
    super().__init__(**func_locals)


  # Container Command
  def set_command(self, command: list) -> Container:
    self.command = command
    return self

  # Environment Variables

  def set_env_variables(self, variables: dict[str]) -> Container:
    self.env_variables = variables
    return self

  def add_env_variable(self, key: str, value: str) -> Container:
    self.env_variables[key] = value
    return self

  # === Volume Stuff ===
  def mount_volume(self, name, mount_path, read_only=False) -> Container:
    self.volume_mounts.append(k8s.VolumeMount(name=name, mount_path=mount_path, read_only=read_only))
    return self

  # === Security ===

  def set_security_context(self, security_context: SecurityContext) -> Container:
    self.security_context = security_context
    return self

  # === Resource Limits and Requests ===
  def set_resource_limit(self, resource_name: str, resource_quantity):
    self.resource_requirements.add_limit(resource_name, resource_quantity)

  def set_resource_request(self, resource_name: str, resource_quantity):
    self.resource_requirements.add_request(resource_name, resource_quantity)

  # === Networking ===

  def add_port(self, name: str, port: int) -> ContainerPort:
    return self.ports.add_port(name=name, port=port)

  def add_ports(self, ports: Mapping[str, int]) -> ContainerPorts:
    for port_name in ports:
      self.ports.add_port(name=port_name, port=ports[port_name])
    return self.ports

  # === Liveness and Readiness ===

  def set_liveness_probe(self, path: str, port: int) -> Container:
    self.liveness_probe_path = path
    self.liveness_probe_port = port
    return self

  def set_readiness_probe(self, path: str, port: int) -> Container:
    self.readiness_probe_path = path
    self.readiness_probe_port = port
    return self

  def render_k8s_resource(self, chart: Chart) -> k8s.Container:

    if self.liveness_probe_path is not None and self.liveness_probe_port is not None:
      liveness_probe = k8s.Probe(http_get=k8s.HttpGetAction(port=self.liveness_probe_port,
                                                            path=self.liveness_probe_path))
    else:
      liveness_probe = None

    if self.readiness_probe_port is not None and self.readiness_probe_port is not None:
      readiness_probe = k8s.Probe(http_get=k8s.HttpGetAction(port=self.readiness_probe_port,
                                                            path=self.readiness_probe_path))
    else:
      readiness_probe = None

    return k8s.Container(name=self.name,
                         image=f"{self.image}:{self.tag}",
                         ports=self.ports.render(),
                         security_context=self.security_context.render(),
                         resources=self.resource_requirements.render(),
                         volume_mounts=self.volume_mounts,
                         command=self.command,
                         liveness_probe=liveness_probe,
                         readiness_probe=readiness_probe)
